@@ -1,8 +1,12 @@
 import base64
 import binascii
+import functools
 import importlib
+import inspect
 import logging
+import re
 import sys
+from typing import Optional, Tuple
 
 import structlog
 from six.moves import urllib
@@ -23,17 +27,17 @@ def get_path_and_address(domain, address):
 
 
 def import_class(module_class_name):
-    module_name, _, class_name = module_class_name.rpartition('.')
+    module_name, _, class_name = module_class_name.rpartition(".")
     module = importlib.import_module(module_name)
     return getattr(module, class_name)
 
 
 def initialize_api(api_config):
-    class_name = api_config['class']
+    class_name = api_config["class"]
     cls = import_class(class_name)
-    params = api_config.get('parameters', None)
+    params = api_config.get("parameters", None)
 
-    if params:
+    if params and inspect.signature(cls).parameters:
         instance = cls(**params)
     else:
         instance = cls()
@@ -50,52 +54,48 @@ def parse_basic_auth_token(token):
         raise InvalidAuthHeader("Can't decode Basic Auth header value")
 
     try:
-        value = value.decode('utf-8')
-        username, password = value.split(':', 1)
+        value = value.decode("utf-8")
+        username, password = value.split(":", 1)
         return (username, password)
     except ValueError:
         raise InvalidAuthHeader("Invalid Basic Auth header value")
 
 
-class PlainRenderer(object):
-
+class PlainRenderer:
     def __call__(self, logger, name, event_dict):
         details = event_dict.copy()
-        timestamp = details.pop('timestamp')
-        logger = details.pop('logger')
-        level = details.pop('level')
-        event = details.pop('event')
-        pairs = ', '.join(['%s=%s' % (k, v) for k, v in details.items()])
-        return (
-            '{timestamp} [{logger}] {level}: {event} {pairs}'
-            .format(
-                timestamp=timestamp,
-                logger=logger,
-                level=level,
-                event=event,
-                pairs=('{{{}}}'.format(pairs) if pairs else "")))
+        timestamp = details.pop("timestamp")
+        logger = details.pop("logger")
+        level = details.pop("level")
+        event = details.pop("event")
+        pairs = ", ".join(["%s=%s" % (k, v) for k, v in details.items()])
+        return "{timestamp} [{logger}] {level}: {event} {pairs}".format(
+            timestamp=timestamp,
+            logger=logger,
+            level=level,
+            event=event,
+            pairs=("{{{}}}".format(pairs) if pairs else ""),
+        )
 
 
 def configure_logging(logging_levels, plain=False, stream=sys.stderr):
-    renderer = (
-        PlainRenderer() if plain else
-        structlog.processors.JSONRenderer())
+    renderer = PlainRenderer() if plain else structlog.processors.JSONRenderer()
 
     attr_processors = [
         structlog.stdlib.add_logger_name,
         structlog.stdlib.add_log_level,
-        structlog.processors.TimeStamper(fmt='iso')
+        structlog.processors.TimeStamper(fmt="iso"),
     ]
 
     structlog.configure_once(
         processors=(
-            [structlog.stdlib.filter_by_level] +
-            attr_processors +
-            [
+            [structlog.stdlib.filter_by_level]
+            + attr_processors
+            + [
                 structlog.stdlib.PositionalArgumentsFormatter(),
                 structlog.processors.StackInfoRenderer(),
                 structlog.processors.format_exc_info,
-                structlog.stdlib.ProcessorFormatter.wrap_for_formatter
+                structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
             ]
         ),
         context_class=dict,
@@ -105,7 +105,8 @@ def configure_logging(logging_levels, plain=False, stream=sys.stderr):
     )
 
     formatter = structlog.stdlib.ProcessorFormatter(
-        processor=renderer, foreign_pre_chain=attr_processors)
+        processor=renderer, foreign_pre_chain=attr_processors
+    )
 
     handler = AtomicStreamHandler(stream)
     handler.setFormatter(formatter)
@@ -119,8 +120,8 @@ def configure_logging(logging_levels, plain=False, stream=sys.stderr):
 
     for logger, level in logging_levels.items():
 
-        if logger.lower() == 'root':
-            logger = ''
+        if logger.lower() == "root":
+            logger = ""
 
         logging.getLogger(logger).setLevel(level.upper())
 
@@ -154,37 +155,38 @@ class AtomicStreamHandler(logging.StreamHandler):
     def emit(self, record):
         try:
             msg = self.format(record)
-            self.stream.write('%s\n' % msg)
+            self.stream.write("%s\n" % msg)
             self.flush()
         except Exception:
             self.handleError(record)
 
 
 def sync_conf_dict_into_db(server, config, force_collection_deletion=False):
-    services = config.get('services', [])
-    sync_services(server, services)
-    collections = config.get('collections', [])
+    services = config.get("services", [])
+    sync_services(server.servers.taxii1, services)
+    collections = config.get("collections", [])
     sync_collections(
-        server, collections, force_deletion=force_collection_deletion)
-    accounts = config.get('accounts', [])
+        server.servers.taxii1, collections, force_deletion=force_collection_deletion
+    )
+    accounts = config.get("accounts", [])
     sync_accounts(server, accounts)
 
 
 def sync_services(server, services):
     manager = server.persistence
 
-    defined_by_id = {s['id']: s for s in services}
+    defined_by_id = {s["id"]: s for s in services}
     existing_by_id = {s.id: s for s in manager.get_services()}
 
     created_counter = 0
     updated_counter = 0
 
     for service in services:
-        existing = existing_by_id.get(service['id'])
+        existing = existing_by_id.get(service["id"])
         if existing:
             properties = service.copy()
-            properties.pop('id')
-            existing.type = properties.pop('type')
+            properties.pop("id")
+            existing.type = properties.pop("type")
             existing.properties = properties
             existing = manager.update_service(existing)
             log.info("sync_services.updated", id=existing.id)
@@ -206,40 +208,39 @@ def sync_services(server, services):
         "sync_services.stats",
         updated=updated_counter,
         created=created_counter,
-        deleted=deleted_counter)
+        deleted=deleted_counter,
+    )
 
 
 def sync_collections(server, collections, force_deletion=False):
     manager = server.persistence
 
-    defined_by_name = {c['name']: c for c in collections}
+    defined_by_name = {c["name"]: c for c in collections}
     existing_by_name = {c.name: c for c in manager.get_collections()}
 
     created_counter = 0
     updated_counter = 0
 
     for collection in collections:
-        existing = existing_by_name.get(collection['name'])
+        existing = existing_by_name.get(collection["name"])
         collection_data = collection.copy()
-        service_ids = collection_data.pop('service_ids')
+        service_ids = collection_data.pop("service_ids")
         if existing:
-            collection_data.pop('id', None)
+            collection_data.pop("id", None)
             bindings = deserialize_content_bindings(
-                collection_data.pop('supported_content', []))
+                collection_data.pop("supported_content", [])
+            )
             for k, v in collection_data.items():
                 setattr(existing, k, v)
             existing.supported_content = bindings
             cobj = manager.update_collection(existing)
             manager.set_collection_services(cobj.id, service_ids)
-            log.info(
-                "sync_collections.updated", name=cobj.name, id=cobj.id)
+            log.info("sync_collections.updated", name=cobj.name, id=cobj.id)
             updated_counter += 1
         else:
-            cobj = manager.create_collection(
-                CollectionEntity(**collection_data))
+            cobj = manager.create_collection(CollectionEntity(**collection_data))
             manager.set_collection_services(cobj.id, service_ids)
-            log.info(
-                "sync_collections.created", name=cobj.name, id=cobj.id)
+            log.info("sync_collections.created", name=cobj.name, id=cobj.id)
             created_counter += 1
 
     disabled_counter = 0
@@ -261,39 +262,42 @@ def sync_collections(server, collections, force_deletion=False):
         updated=updated_counter,
         created=created_counter,
         disabled=disabled_counter,
-        deleted=deleted_counter)
+        deleted=deleted_counter,
+    )
 
 
 def sync_accounts(server, accounts):
     manager = server.auth
 
-    defined_by_username = {a['username']: a for a in accounts}
+    defined_by_username = {a["username"]: a for a in accounts}
     existing_by_username = {a.username: a for a in manager.get_accounts()}
 
     created_counter = 0
     updated_counter = 0
     for account in accounts:
-        existing = existing_by_username.get(account['username'])
+        existing = existing_by_username.get(account["username"])
         if existing:
             properties = account.copy()
-            password = properties.pop('password')
-            existing.permissions = properties.get('permissions', {})
-            existing.is_admin = properties.get('is_admin', False)
+            password = properties.pop("password")
+            existing.permissions = properties.get("permissions", {})
+            existing.is_admin = properties.get("is_admin", False)
             existing = manager.update_account(existing, password)
             log.info("sync_accounts.updated", username=existing.username)
             updated_counter += 1
         else:
             obj = Account(
                 id=None,
-                username=account['username'],
-                permissions=account.get('permissions', {}),
-                is_admin=account.get('is_admin', False))
-            obj = manager.update_account(obj, account['password'])
+                username=account["username"],
+                permissions=account.get("permissions", {}),
+                is_admin=account.get("is_admin", False),
+            )
+            obj = manager.update_account(obj, account["password"])
             log.info("sync_accounts.created", username=obj.username)
             created_counter += 1
     deleted_counter = 0
-    missing_usernames = (
-        set(existing_by_username.keys()) - set(defined_by_username.keys()))
+    missing_usernames = set(existing_by_username.keys()) - set(
+        defined_by_username.keys()
+    )
     for username in missing_usernames:
         manager.delete_account(username)
         deleted_counter += 1
@@ -303,4 +307,44 @@ def sync_accounts(server, accounts):
         "sync_accounts.stats",
         updated=updated_counter,
         created=created_counter,
-        deleted=deleted_counter)
+        deleted=deleted_counter,
+    )
+
+
+def register_handler(
+    url_re: str,
+    valid_methods: Optional[Tuple[str]] = None,
+    valid_accept_mimetypes: Optional[Tuple[str]] = None,
+    valid_content_types: Optional[Tuple[str]] = None,
+):
+    """
+    Register decorated method as handler function for `url_re`.
+
+    :param str url_re: The regex to trigger the handler on
+    :param list valid_methods: The list of methods to accept for this handler, defaults to ("GET",)
+    :param list valid_accept_mimetypes:
+        The list of accepted mimetypes to accept for this handler, defaults to
+        ("application/taxii+json;version=2.1",)
+    :param list valid_content_types:
+        The list of content types to accept for this handler, defaults to
+        ("application/json",)
+    """
+    if valid_methods is None:
+        valid_methods = ("GET",)
+    if valid_accept_mimetypes is None:
+        valid_accept_mimetypes = ("application/taxii+json;version=2.1",)
+    if valid_content_types is None:
+        valid_content_types = ("application/json",)
+
+    def inner_decorator(method):
+        @functools.wraps(method)
+        def inner(*args, **kwargs):
+            return method(*args, **kwargs)
+
+        inner.registered_url_re = re.compile(url_re)
+        inner.registered_valid_methods = valid_methods
+        inner.registered_valid_accept_mimetypes = valid_accept_mimetypes
+        inner.registered_valid_content_types = valid_content_types
+        return inner
+
+    return inner_decorator
